///////////////////////////////////////////////////////////////////////////////
// PjModel class
//
// Last update: 23/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;
import java.io.BufferedReader;
import java.applet.AudioClip;

import jv.geom.PgElementSet;
import jv.geom.PgPolygonSet;
import jv.loader.PgJvxLoader;
import jv.number.PuDouble;
import jv.object.PsDebug;
import jv.object.PsUtil;
import jv.project.PgJvxSrc;
import jv.project.PjProject;
import jv.project.PvCameraIf;
import jv.project.PvDisplayIf;
import jv.vecmath.PdVector;
import jvx.geom.PwCleanMesh;
import jvx.geom.PwIntersectWithFunction;
import jvx.geom.PwTransform;
import jvx.loader.PgOffLoader;



///////////////////////////////////////////////////////////////////////////////
// Model applet
///////////////////////////////////////////////////////////////////////////////
public class PjModel extends PjProject 
{
	protected int idiom;
	protected String codebase;
	protected double initial_transparency;

	protected PuDouble transparency;
	protected PuDouble perspective;

	protected int numberOfVertices;
	protected int numberOfEdges;
	protected int numberOfElements;

	protected PgElementSet poly;
	protected PgElementSet copy;

	protected PvDisplayIf disp;
    protected PvCameraIf cam;

	protected PgElementSet planeXY;
	protected PgElementSet planeXZ;
	protected PgElementSet planeYZ;
	protected double xMin;
	protected double xMax;
	protected double yMin;
	protected double yMax;
	protected double zMin;
	protected double zMax;
	protected double delta;
	protected double xO;
	protected double yO;
	protected double zO;    	
	
	//////////////////////////////////////////////////////////////////////////
	// Default constructor
	//////////////////////////////////////////////////////////////////////////
	public PjModel(String input_codebase, 
			       int input_idiom,  
			       double input_initial_transparency) 
	{
		super("Spatial Geometry Applet");

		codebase = input_codebase;
	    
		idiom = input_idiom;
		initial_transparency = input_initial_transparency;

		if (getClass() == PjModel.class) 
		{
			init();
		};
	} // End of PjModel()

	// ////////////////////////////////////////////////////////////////////////
	// Initialize variables
	// ////////////////////////////////////////////////////////////////////////
	public void init() 
	{
		poly = new PgElementSet(3);

		planeXY = new PgElementSet(3);
		planeXY.setNumElements(1);
		planeXY.setNumVertices(4);
		planeXZ = new PgElementSet(3);
		planeXZ.setNumElements(1);
		planeXZ.setNumVertices(4);
		planeYZ = new PgElementSet(3);
		planeYZ.setNumElements(1);
		planeYZ.setNumVertices(4);

		copy = new PgElementSet(3);

		// Transparency slider
		transparency = new PuDouble("Transparncia:", this);
		transparency.setDefBounds(0.0, 1.0, 0.01, 0.05);
		transparency.setDefValue(initial_transparency);
		transparency.init();
		transparency.setEnabled(true);
		
		// Perspective slider
		perspective = new PuDouble("Perspectiva:", this);
		perspective.setDefBounds(0.0, 1.5, 0.01, 0.05);
		perspective.setDefValue(0.5);
		perspective.init();
		perspective.setEnabled(true);		
	}; // End of init()

	// ////////////////////////////////////////////////////////////////////////
	// Load geometry file
	// ////////////////////////////////////////////////////////////////////////
	public void loadGeometryFromFile(Model object) 
	{		
		
		String filename = object.getFileName();
		String filetype = object.getFileType();
		String pathname = codebase + filename + "." + filetype;

		/*
			URL fullpath = new URL(pathname);
			java.io.InputStream r = fullpath.openStream();
			BufferedReader in = new BufferedReader(new InputStreamReader(r));
			*/
			BufferedReader in = PsUtil.open(pathname);
			if (in == null) {
				PsDebug.warning("could not open file  " + pathname + "...");
				System.out.println("could not open file  " + pathname + "...");
				return;
			} 

				// Loading polyhedron
				PgJvxSrc[] geometries;
				if (filetype.equals("off")) 
				{
					PgOffLoader offloader = new PgOffLoader();
					offloader.load(in);
					geometries = offloader.getGeometries();
					poly.setJvx(geometries[0]);
				}
				if (filetype.equals("jvx")) 
				{
					PgJvxLoader jvxloader = new PgJvxLoader();
					jvxloader.load(in);
					geometries = jvxloader.getGeometries();
					poly.setJvx(geometries[0]);
				}

				// Configuring the polyhedron
				poly.setTransparency(initial_transparency);
				if (transparency.getValue() > 0.05) 
				{
					poly.showTransparency(true);
				} 
				else 
				{
					poly.showTransparency(false);
				}
				poly.showElementColors(true);
				poly.showVertices(object.isShowingVertices());				
				poly.showEdges(object.isShowingEdges());
				poly.showElements(object.isShowingElements());				
				poly.showVertexOutline(true);
				poly.setGlobalEdgeColor(Color.BLACK);
				poly.setGlobalVertexColor(Color.WHITE);
				poly.showVertexColors(true);

				PwCleanMesh.identifyVertices(poly, 1E-010D);
				poly.makeNeighbour();
				poly.close();
				
				xMin = (poly.getBounds())[0].getEntry(0);
				yMin = (poly.getBounds())[0].getEntry(1);
				zMin = (poly.getBounds())[0].getEntry(2);
				xMax = (poly.getBounds())[1].getEntry(0);
				yMax = (poly.getBounds())[1].getEntry(1);
				zMax = (poly.getBounds())[1].getEntry(2);

				double deltaX = xMax - xMin;
				double deltaY = yMax - yMin;
				double deltaZ = zMax - zMin;

				delta = deltaZ;
				if (delta < deltaX) 
				{
					delta = deltaX;
				}
				if (delta < deltaY) 
				{
					delta = deltaY;
				}
				delta = 1.1 * Math.sqrt(3.0) * delta;

				xO = xMin - (delta - deltaX) / 2.0;
				yO = yMin - (delta - deltaY) / 2.0;
				zO = zMin - (delta - deltaZ) / 2.0;

				// Plane XY
				planeXY.setVertex(0, xO, yO, zO);
				planeXY.setVertex(1, xO, yO + delta, zO);
				planeXY.setVertex(2, xO + delta, yO + delta, zO);
				planeXY.setVertex(3, xO + delta, yO, zO);
				planeXY.setElement(0, 0, 1, 2, 3);
				planeXY.showElementColors(true);
				planeXY.showElements(true);
				planeXY.showEdges(true);
				planeXY.setGlobalElementColor(Color.LIGHT_GRAY);
				planeXY.setTransparency(0.7);
				planeXY.showTransparency(true);

				// Plane XZ
				planeXZ.setVertex(0, xO, yO, zO);
				planeXZ.setVertex(1, xO + delta, yO, zO);
				planeXZ.setVertex(2, xO + delta, yO, zO + delta);
				planeXZ.setVertex(3, xO, yO, zO + delta);
				planeXZ.setElement(0, 0, 1, 2, 3);
				planeXZ.showElementColors(true);
				planeXZ.showElements(true);
				planeXZ.showEdges(true);
				planeXZ.setGlobalElementColor(Color.LIGHT_GRAY);
				planeXZ.setTransparency(0.7);
				planeXZ.showTransparency(true);

				// Plane YZ
				planeYZ.setVertex(0, xO, yO, zO);
				planeYZ.setVertex(1, xO, yO + delta, zO);
				planeYZ.setVertex(2, xO, yO + delta, zO + delta);
				planeYZ.setVertex(3, xO, yO, zO + delta);
				planeYZ.setElement(0, 0, 1, 2, 3);
				planeYZ.showElementColors(true);
				planeYZ.showElements(true);
				planeYZ.showEdges(true);
				planeYZ.setGlobalElementColor(Color.LIGHT_GRAY);
				planeYZ.setTransparency(0.7);
				planeYZ.showTransparency(true);

				poly.update(poly); // Don't forget to update!

				planeXY.update(planeXY);
				planeXZ.update(planeXZ);
				planeYZ.update(planeYZ);

				PwCleanMesh.identifyVertices(poly, 1E-010D);
				poly.makeNeighbour();

				poly.setTransparency(initial_transparency);
				poly.showTransparency(true);
				poly.update(poly);
				
				PwCleanMesh.identifyVertices(copy, 1E-010D);
				copy.makeNeighbour();

				copy.setTransparency(initial_transparency);
				copy.showTransparency(true);
				copy.update(copy);				
								
				// Copy of poly
				copy.copy(poly);
	} // End of loadGeometryFromFile()


	// ////////////////////////////////////////////////////////////////////////
	// Things to do when the class is started
	// ////////////////////////////////////////////////////////////////////////
	public void start() 
	{
		super.start();

		// Necessary to apply PwWorkshop operations
		PwCleanMesh.identifyVertices(poly, 1E-010D);
		poly.makeNeighbour();

		poly.setTransparency(initial_transparency);
		poly.showTransparency(true);
		// addGeometry(poly);
		// addGeometry(planeXY);
		// addGeometry(planeXZ);
		// addGeometry(planeYZ);
		selectGeometry(poly);

		update(this);

		disp = getDisplay(); // The PwUnfold needs a reference to the
		                     // display in order to work properly.

		PdVector d1 = new PdVector(1, 1, 1);
		PdVector d2 = new PdVector(3);
		d2.copy(d1);
		d2.multScalar(-1);
		cam = disp.getCamera();
		cam.setPosition(d1);
		cam.setViewDir(d2);
		cam.setRoll(0);
		cam.update(cam);
		disp.fit();		
	} // End of start()

	// ///////////////////////////////////////////////////////////////////////
	// Update the class whenever a child has changed. Method is usually
	// invoked from the children.
	// ///////////////////////////////////////////////////////////////////////
	public boolean update(Object event) 
	{
		if (event == null)
		{
			return (true);
		};

		// Solid transparency
		if (event == transparency) 
		{
			poly.setTransparency(transparency.getValue());
			copy.setTransparency(transparency.getValue());

			if (transparency.getValue() > 0.05)
			{
				poly.showTransparency(true);
				copy.showTransparency(true);
			} 
			else 
			{
				poly.showTransparency(false);
				copy.showTransparency(false);
			}

			poly.update(poly);

			return (true);
		};
		
		// Perspective
		if (event == perspective) 
		{
            cam.setFieldOfView(perspective.getValue());
            cam.update(cam);
			return (true);
		};
		

/*		
		// Rotation
		if (event == rot1 || event == rot2 || event == rot3) 
		{
			double angle1 = rot1.getValue() * Math.PI / 180.0;
			double angle2 = rot2.getValue() * Math.PI / 180.0;
			double angle3 = rot3.getValue() * Math.PI / 180.0;

			poly.copy(original);

			PwTransform.rotate(poly, angle1, 0);
			PwTransform.rotate(poly, angle2, 1);
			PwTransform.rotate(poly, angle3, 2);

			poly.update(poly);
			
			copy.copy(poly);
			
			m_pwIntersectWithFunction.setGeometry(copy);			   
	  	    m_pwIntersectWithFunction.setFunction("w");				
			
			updateCutPlane();

			return (true);
		}
 */		
		
		return (super.update(event));
	} // End of update()		       
} // End of PjModel classs
