///////////////////////////////////////////////////////////////////////////////
// PjModel Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Button;
import java.awt.Choice;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;


///////////////////////////////////////////////////////////////////////////////
// Info panel for graph model applet.
///////////////////////////////////////////////////////////////////////////////
public class PjModel_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjModel m_pjModel;

   protected int idiom = 0;  

   protected Choice poly_category;
   protected Choice poly_name;

   protected Panel m_aPanel; // Transparency panel
   protected Panel m_pPanel; // Perspective panel
   
   protected String[] title_trans = {"Tomografia",
                                     "Tomography"};
   

   protected String[] vis_transparency_trans = {"Transparncia:",
                                                "Transparency:"};
   protected String[] vis_perspective_trans = {"Perspectividade:",
                                               "Perspectivity:"};
   
   protected String[] vis_vertices_trans = {"Exibir vrtices",
                                            "Show vertices"};
   protected String[] vis_edges_trans = {"Exibir arestas", "Show edges"};
   protected String[] vis_elements_trans = {"Exibir faces", "Show faces"};

   protected String[] vis_panel_trans = {"Visualizao", "Visualize"};
   
   protected String[] cut_translation_trans = {"Altura do plano:", "Height:"};   
   
 
   protected String[] polyhedra_trans =
                 {"Poliedros", "Polyhedra"};
   protected String[] nonpolyhedral_surfaces_trans =
                 {"Superfcies No Polidricas", "Nonpolyhedral Surfaces"};      
   protected String[] computer_graphics_models_trans =
                 {"Modelos 3D de Computao Grfica", "Computer Graphics 3D Models"};   
   

   protected String[] of_trans = {"de", "of"};   

   protected Category polyhedra;
   protected Category nonpolyhedral_surfaces;   
   protected Category computer_graphics_models;
   
   protected Model model;
   

   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjModel_IP()
   {
      super();

      if (getClass() == PjModel_IP.class)
      {
         init();
      };
   }; // End of PjModel_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      // addTitle(title_trans[idiom]);

      add(new Label(" "));

      poly_category = new Choice();
      add(poly_category);
      poly_category.addItemListener(this);

      poly_name = new Choice();
      add(poly_name);
      poly_name.addItemListener(this);

      add(new Label(" "));
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjModel = (PjModel) parent;
      idiom = m_pjModel.idiom;

      // Category and name of the solid
      poly_category.add(polyhedra_trans[idiom]);
      poly_category.add(nonpolyhedral_surfaces_trans[idiom]);
      poly_category.add(computer_graphics_models_trans[idiom]);
      
      
      poly_category.select(polyhedra_trans[idiom]);
      buildDataBase();
      buildChoice(polyhedra_trans[idiom]);
      poly_name.select(polyhedra.getModel(0).getName(idiom));
            
      // Visualization panel
      PsPanel visPanel = new PsPanel();
      
      visPanel.add(new Label(" "));
            
      visPanel.addLine(1);
      
      m_aPanel = new Panel();
      m_aPanel.setLayout(new GridLayout(1, 1));
      visPanel.add(m_aPanel);
      m_aPanel.removeAll();
      m_pjModel.transparency.setName(vis_transparency_trans[idiom]);
      m_aPanel.add(m_pjModel.transparency.newInspector(PsPanel.INFO_EXT));

      m_pPanel = new Panel();
      m_pPanel.setLayout(new GridLayout(1, 1));
      visPanel.add(m_pPanel);
      m_pPanel.removeAll();
      m_pjModel.perspective.setName(vis_perspective_trans[idiom]);
      m_pPanel.add(m_pjModel.perspective.newInspector(PsPanel.INFO_EXT));
      
      visPanel.addLine(1);

      // tabPanel.addPanel(vis_panel_trans[idiom], visPanel);

      // Validate
      // tabPanel.validate();
      
      buildChoice(polyhedra_trans[idiom]);   
      
      m_pjModel.loadGeometryFromFile(polyhedra.getModel(0));
      m_pjModel.addGeometry(m_pjModel.poly);

      super.setParent(parent);
   }; // End of setParent()
   

   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (m_pjModel == event)
      {
         // Fill here!

         return(true);
      };

      return(super.update(event));
   }; // End of update()
   
   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjModel == null)
      {
          return;
      };
         
      Object source = event.getSource();

      
   }; // End of actionPerformed()

   
   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjModel == null)
       {
          return;
       };

       Object source = event.getSource();

       //////////////////////////////////////////////////////////////////////
       // Names
       //////////////////////////////////////////////////////////////////////
       if (source == poly_category)
       {        
    	  buildChoice(poly_category.getSelectedItem());    
    	  poly_name.select(0);
    	   	
          loadModel(poly_category.getSelectedItem(), poly_name.getSelectedItem());
       }
       
       if (source == poly_name)
       {
          loadModel(poly_category.getSelectedItem(), poly_name.getSelectedItem());
       }       
    }; // End of itemStateChanged()


    
    //////////////////////////////////////////////////////////////////////////
    // Load model
    //////////////////////////////////////////////////////////////////////////
    public void loadModel(String category, String solid)
    {
 	  if (category.equals(polyhedra_trans[idiom]))
 	  {
 		  for (int i = 0; i < polyhedra.getNumberOfModels(); i++)
 		  {
 			  if (polyhedra.getModel(i).getName(idiom).equals(solid))
 			  {
 				  model = polyhedra.getModel(i);
 				  break;
 			  }
 		  }
 	  }

 	  
 	  if (category.equals(nonpolyhedral_surfaces_trans[idiom]))
 	  {
 		  for (int i = 0; i < nonpolyhedral_surfaces.getNumberOfModels(); i++)
 		  {
 			  if (nonpolyhedral_surfaces.getModel(i).getName(idiom).equals(solid))
 			  {
 				  model = nonpolyhedral_surfaces.getModel(i);
 				  break;
 			  }
 		  }
 	  }

 	  
 	  if (category.equals(computer_graphics_models_trans[idiom]))
 	  {
 		  for (int i = 0; i < computer_graphics_models.getNumberOfModels(); i++)
 		  {
 			  if (computer_graphics_models.getModel(i).getName(idiom).equals(solid))
 			  {
 				  model = computer_graphics_models.getModel(i);
 				  break;
 			  }
 		  }
 	  }
 	   	  
       m_pjModel.loadGeometryFromFile(model);
       m_pjModel.disp.fit();
    }    

    //////////////////////////////////////////////////////////////////////////
    // Build choice of models within a given category
    //////////////////////////////////////////////////////////////////////////
    protected void buildChoice(String c)
    {
        poly_name.removeAll();
        if (c.equals(polyhedra_trans[idiom]))
        {
            for (int i = 0; i < polyhedra.getNumberOfModels(); i++)
            {
               poly_name.add(polyhedra.getModel(i).getName(idiom));
            }
        }

        if (c.equals(nonpolyhedral_surfaces_trans[idiom]))
        {
                for (int i = 0; i < nonpolyhedral_surfaces.getNumberOfModels(); i++)
                {
                   poly_name.add(nonpolyhedral_surfaces.getModel(i).getName(idiom));
                }
        }
                
        if (c.equals(computer_graphics_models_trans[idiom]))
        {
            for (int i = 0; i < computer_graphics_models.getNumberOfModels(); i++)
            {
               poly_name.add(computer_graphics_models.getModel(i).getName(idiom));
            }
        }                
    }; // End of buildChoice()


    //////////////////////////////////////////////////////////////////////////
    // Build database of solids
    //////////////////////////////////////////////////////////////////////////
    protected void buildDataBase()
    {
        int i;

        // POLYHEDRA
        i = 0;
        polyhedra = new Category(polyhedra_trans, 8);

        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Tetraedro Regular",
                 "Regular Tetrahedron"},
                "off",
                "platonic-tetrahedron",
                false, true, true,
                true,
                true));
        i++;

        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Cubo",
                 "Cube"},
                "off",
                "platonic-cube",
                false, true, true,
                true,
                true));
        i++;

        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Octaedro Regular",
                 "Regular Octahedron"},
                "off",
                "platonic-octahedron",
                false, true, true,
                true,
                true));
        i++;
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Dodecaedro Regular",
                 "Regular Dodecahedron"},
                "off",
                "platonic-dodecahedron",
                false, true, true,
                true,
                true));
        i++;
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Icosaedro Regular",
                 "Regular Icosahedron"},
                "off",
                "platonic-icosahedron",
                false, true, true,
                true,
                true));
        i++;                
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Pirmide de Base Quadrangular",
                 "Pyramid with Quadrangular Base"},
                "off",
                "pyramids-regular-quadrangular",
                false, true, true,
                true,
                true));
        i++;        

        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Pirmide de Base Pentagonal",
                 "Pyramid with Pentagonal Base"},
                "off",
                "pyramids-regular-pentagonal",
                false, true, true,
                true,
                true));
        i++;                
        
        polyhedra.addModel(i, new Model(polyhedra_trans,
                new String[]
                {"Prisma Regular Pentagonal",
                 "Regular Pentagonal Prism"},
                "off",
                "prisms-regular-pentagonal",
                false, true, true,
                true,
                true));
        i++;

       // CURVILINEAR SHAPES
       i = 0;
       nonpolyhedral_surfaces = new Category(nonpolyhedral_surfaces_trans, 4);

       nonpolyhedral_surfaces.addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Cilindro Reto",
               "Right Cylinder"},
              "off",
              "surfaces-cylinder",
              false, false, true,
              true,
              true));
      i++;
      
      nonpolyhedral_surfaces .addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Cone Reto",
               "Right Cone"},
              "off",
              "surfaces-cone",
              false, false, true,
              true,
              true));
      i++;
            
      nonpolyhedral_surfaces .addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Esfera",
               "Sphere"},
              "off",
              "surfaces-sphere",
              false, false, true,
              true,
              true));
      i++;
            
      nonpolyhedral_surfaces .addModel(i, new Model(nonpolyhedral_surfaces_trans,
              new String[]
              {"Toro",
               "Torus"},
              "off",
              "surfaces-torus",
              false, false, true,
              true,
              true));
      i++;      
      
       
      // MISCELLANY 
      i = 0;
      computer_graphics_models = new Category(computer_graphics_models_trans, 4);

      computer_graphics_models.addModel(i, new Model(computer_graphics_models_trans,
              new String[]
              {"Crnio",
               "Skull"},
              "off",
              "miscellany-skull",
              false, false, true,
              true,
              true));
      computer_graphics_models.getModel(i).setImproveUnfolding(true, 2);
      i++;
            
      computer_graphics_models.addModel(i, new Model(computer_graphics_models_trans,
               new String[]
               {"Dromedrio",
                "Dromedary"},
               "off",
               "miscellany-dromedary",
               false, false, true,
               true,
               true));
      computer_graphics_models.getModel(i).setImproveUnfolding(true, 6);
      i++;

      computer_graphics_models.addModel(i, new Model(computer_graphics_models_trans,
               new String[]
               {"Polvo",
                "Octopus"},
               "off",
               "miscellany-octopus",
               false, false, true,
               true,
               true));
       computer_graphics_models.getModel(i).setImproveUnfolding(true, 2);
       i++;
       
       computer_graphics_models.addModel(i, new Model(computer_graphics_models_trans,
               new String[]
               {"Vaca",
                "Cow"},
               "off",
               "miscellany-cow",
               false, false, true,
               true,
               true));
       computer_graphics_models.getModel(i).setImproveUnfolding(true, 6);
      i++;       
    }; // End of buildDataBase()
}; // End of PjModel_IP class
