///////////////////////////////////////////////////////////////////////////////
// PjShowcase Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;


///////////////////////////////////////////////////////////////////////////////
// Info panel for graph showcase applet.
///////////////////////////////////////////////////////////////////////////////
public class PjShowcase_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjShowcase m_pjShowcase;

   protected int idiom = 0;

   protected Choice poly_category;
   protected Choice poly_name;   
   
   protected Checkbox show_object;   

   protected Panel m_aPanel; // Transparency panel
   protected Panel m_pPanel; // Perspective panel
   
   protected Panel m_tPanel;   
   
   protected String[] title_trans = {"Tomografia",
                                     "Showcase"};
   
   protected String[] vis_transparency_trans = {"Transparncia:",
                                                "Transparency:"};
   protected String[] vis_perspective_trans = {"Perspectividade:",
                                               "Perspectivity:"};
   
   protected String[] vis_vertices_trans = {"Exibir vrtices",
                                            "Show vertices"};
   protected String[] vis_edges_trans = {"Exibir arestas", "Show edges"};
   protected String[] vis_elements_trans = {"Exibir faces", "Show faces"};

   protected String[] vis_panel_trans = {"Visualizao", "Visualize"};
   
   protected String[] cut_translation_trans = {"Altura do plano:", "Height:"};     
   
   protected String[] of_trans = {"de", "of"};
   
   protected String[] ruled_surfaces_trans = {"Superfcies Regradas", "Ruled Surfaces"};
   protected String[] terrains_trans = {"Terrenos e Curvas de Nvel", "Terrains and Level Sets"};
   protected String[] conic_sections_trans = {"Sees Cnicas", "Conic Sections"};
   protected String[] show_object_trans = {"Exibir objeto", "Show object"};
   
   protected Category ruled_surfaces;
   protected Category terrains;
   protected Category conic_sections;

   protected Model model;   
   
   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjShowcase_IP()
   {
      super();

      if (getClass() == PjShowcase_IP.class)
      {
         init();
      };
   }; // End of PjShowcase_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      // addTitle(title_trans[idiom]);

      add(new Label(" "));

      poly_category = new Choice();
      add(poly_category);
      poly_category.addItemListener(this);

      poly_name = new Choice();
      add(poly_name);
      poly_name.addItemListener(this);
      

      add(new Label(" "));
      
      // add(new Label(" "));
      show_object = new Checkbox("");
      add(show_object);
      show_object.addItemListener(this);
      
      add(new Label(" "));      
      
      addLine(1);
      
      m_tPanel = new Panel();
      add(m_tPanel);
      
      addLine(1);        
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjShowcase = (PjShowcase) parent;
      idiom = m_pjShowcase.idiom;
      
      // Checkbox translation
      show_object.setLabel(show_object_trans[idiom]);
      
      // Category and name of the solid
      poly_category.add(ruled_surfaces_trans[idiom]);
      poly_category.add(terrains_trans[idiom]);
      poly_category.add(conic_sections_trans[idiom]);

      poly_category.select(m_pjShowcase.category);
      buildDataBase();
      buildChoice(m_pjShowcase.category);
      poly_name.select(m_pjShowcase.solid);
      loadModel(m_pjShowcase.category, m_pjShowcase.solid);

      // Visualization panel
      PsPanel visPanel = new PsPanel();
      
      visPanel.add(new Label(" "));
            
      visPanel.addLine(1);
      
      m_aPanel = new Panel();
      m_aPanel.setLayout(new GridLayout(1, 1));
      visPanel.add(m_aPanel);
      m_aPanel.removeAll();
      m_pjShowcase.transparency.setName(vis_transparency_trans[idiom]);
      m_aPanel.add(m_pjShowcase.transparency.newInspector(PsPanel.INFO_EXT));

      m_pPanel = new Panel();
      m_pPanel.setLayout(new GridLayout(1, 1));
      visPanel.add(m_pPanel);
      m_pPanel.removeAll();
      m_pjShowcase.perspective.setName(vis_perspective_trans[idiom]);
      m_pPanel.add(m_pjShowcase.perspective.newInspector(PsPanel.INFO_EXT));
      
      visPanel.addLine(1);

      // tabPanel.addPanel(vis_panel_trans[idiom], visPanel);

      // Translation panel
      m_tPanel.setLayout(new GridLayout(1, 1));
      m_tPanel.removeAll();
      m_pjShowcase.translation_factor.setName(cut_translation_trans[idiom]);
      m_tPanel.add(m_pjShowcase.translation_factor.newInspector(PsPanel.INFO_EXT));            
            
      super.setParent(parent);
   }; // End of setParent()
   

   //////////////////////////////////////////////////////////////////////////
   // Load model
   //////////////////////////////////////////////////////////////////////////
   public void loadModel(String category, String solid)
   {
	  if (category.equals(ruled_surfaces_trans[idiom]))
	  {
		  for (int i = 0; i < ruled_surfaces.getNumberOfModels(); i++)
		  {
			  if (ruled_surfaces.getModel(i).getName(idiom).equals(solid))
			  {
				  model = ruled_surfaces.getModel(i);
				  break;
			  }
		  }
	  }

	  if (category.equals(terrains_trans[idiom]))
	  {
		  for (int i = 0; i < terrains.getNumberOfModels(); i++)
		  {
			  if (terrains.getModel(i).getName(idiom).equals(solid))
			  {
				  model = terrains.getModel(i);
				  break;
			  }
		  }
	  }
	  
	  if (category.equals(conic_sections_trans[idiom]))
	  {
		  for (int i = 0; i < conic_sections.getNumberOfModels(); i++)
		  {
			  if (conic_sections.getModel(i).getName(idiom).equals(solid))
			  {
				  model = conic_sections.getModel(i);
				  break;
			  }
		  }
	  }	  
	  
      m_pjShowcase.loadGeometryFromFile(model);      
      m_pjShowcase.updateCutPlane();
   }

   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (m_pjShowcase == event)
      {
         // Fill here!

         return(true);
      };

      return(super.update(event));
   }; // End of update()
   
   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjShowcase == null)
      {
          return;
      };
         
      Object source = event.getSource();

   }; // End of actionPerformed()

   
   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjShowcase == null)
       {
          return;
       };

       Object source = event.getSource();
       
       if (source == show_object)
       {
    	   if (show_object.getState() == true)
    	   {
    		   m_pjShowcase.addGeometry(m_pjShowcase.poly);
    	   }
    	   else
    	   {
    		   m_pjShowcase.removeGeometry(m_pjShowcase.poly);
    	   }
       }
       
       if (source == poly_category)
       {
          String category = poly_category.getSelectedItem();
          buildChoice(category);
          poly_name.select(0);
          loadModel(category, poly_name.getSelectedItem());
       }
       
       if (source == poly_name)
       {
          loadModel(poly_category.getSelectedItem(),
        		    poly_name.getSelectedItem());
       }       
    }; // End of itemStateChanged()


    //////////////////////////////////////////////////////////////////////////
    // Build database of solids
    //////////////////////////////////////////////////////////////////////////
    protected void buildDataBase()
    {
        int i;

        // RULED SURFACES
        i = 0;
        ruled_surfaces = new Category(ruled_surfaces_trans, 4);

        ruled_surfaces.addModel(i, new Model(ruled_surfaces_trans,
                  new String[]
                  {"Parabolide hiperblico (sela de cavalo)",
                   "Hyperbolic Paraboloid (horse's saddle)"},
                   "jvx",
                   "objects-horse-saddle",
                   false, false, true,
                   true,
                   true));
        i++;

        ruled_surfaces.addModel(i, new Model(ruled_surfaces_trans,
                  new String[]
                  {"Helicide",
                   "Helicoid"},
                   "jvx",
                   "objects-helicoid",
                   false, false, true,
                   true,
                   true));
        i++;        
        
        ruled_surfaces.addModel(i, new Model(ruled_surfaces_trans,
                  new String[]
                  {"Plano",
                   "Plane"},
                   "jvx",
                   "objects-plane",
                   false, false, true,
                   true,
                   true));
        i++;
        
        ruled_surfaces.addModel(i, new Model(ruled_surfaces_trans,
                  new String[]
                  {"Superfcie cilndrica",
                   "Cylindrical surface"},
                   "jvx",
                   "objects-cylindrical-surface",
                   false, false, true,
                   true,
                   true));
        i++;


        // TERRAINS
        i = 0;
        terrains = new Category(terrains_trans, 4);
        
        terrains.addModel(i, new Model(terrains_trans,
                  new String[]
                  {"Terreno 1",
                   "Terrain 1"},
                   "jvx",
                   "objects-terrain-1",
                   false, false, true,
                   true,
                   true));
        i++;        

        terrains.addModel(i, new Model(terrains_trans,
                new String[]
                {"Terreno 2",
                 "Terrain 2"},
                 "jvx",
                 "objects-terrain-2",
                 false, false, true,
                 true,
                 true));
        i++;        

        terrains.addModel(i, new Model(terrains_trans,
                new String[]
                {"Terreno 3",
                 "Terrain 3"},
                 "jvx",
                 "objects-terrain-3",
                 false, false, true,
                 true,
                 true));
        i++;        
        
        terrains.addModel(i, new Model(terrains_trans,
                new String[]
                {"Terreno 4",
                 "Terrain 4"},
                 "jvx",
                 "objects-terrain-4",
                 false, false, true,
                 true,
                 true));
        i++;        

        // CONIC SECTIONS
        i = 0;
        conic_sections = new Category(conic_sections_trans, 4);
        
        conic_sections.addModel(i, new Model(conic_sections_trans,
                  new String[]
                  {"Sees Cnicas 1",
                   "Conic Sections 1"},
                   "jvx",
                   "objects-conic-sections-1",
                   false, false, true,
                   true,
                   true));
        i++;        

        conic_sections.addModel(i, new Model(conic_sections_trans,
                new String[]
                {"Sees Cnicas 2",
                 "Conic Sections 2"},
                 "jvx",
                 "objects-conic-sections-2",
                 false, false, true,
                 true,
                 true));
        i++;        
        
        conic_sections.addModel(i, new Model(conic_sections_trans,
                new String[]
                {"Sees Cnicas 3",
                 "Conic Sections 3"},
                 "jvx",
                 "objects-conic-sections-3",
                 false, false, true,
                 true,
                 true));
        i++;                
        
        conic_sections.addModel(i, new Model(conic_sections_trans,
                new String[]
                {"Sees Cnicas 4",
                 "Conic Sections 4"},
                 "jvx",
                 "objects-conic-sections-4",
                 false, false, true,
                 true,
                 true));
        i++;                
        
        
    }; // End of buildDataBase()

    
    //////////////////////////////////////////////////////////////////////////
    // Build choice of models within a given category
    //////////////////////////////////////////////////////////////////////////
    protected void buildChoice(String c)
    {
        poly_name.removeAll();
        if (c.equals(ruled_surfaces_trans[idiom]))
        {
            for (int i = 0; i < ruled_surfaces.getNumberOfModels(); i++)
            {
               poly_name.add(ruled_surfaces.getModel(i).getName(idiom));
            }
        }

        if (c.equals(terrains_trans[idiom]))
        {
            for (int i = 0; i < terrains.getNumberOfModels(); i++)
            {
               poly_name.add(terrains.getModel(i).getName(idiom));
            }
        }

        if (c.equals(conic_sections_trans[idiom]))
        {
            for (int i = 0; i < conic_sections.getNumberOfModels(); i++)
            {
               poly_name.add(conic_sections.getModel(i).getName(idiom));
            }
        }
        
        
    }; // End of buildChoice()
    
}; // End of PjShowcase_IP class
